/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */

package searchtool;

import searchtool.components.*;
import searchtool.wrappers.*;
import searchtool.util.*;
import searchtool.halflife.wrappers.*;
import javax.swing.*;
import searchtool.actions.*;
import searchtool.gamespy.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author thorsten
 *
 */
public class GameController {

	public final static String mainListButtonFileName = "mainlist.button",
		favouriteListButtonFileName = "favouritelist.button",
		blacklistButtonFileName = "blacklist.button",
		lastConnectedListButtonFileName = "lastconnectedlist.button",
		resultListButtonFileName = "resultlist.button",
		rulesConfigFileName = "rules.conf",
		serverFileName = "serverfile",
		configFileName = "game.conf";

	public final String PING_TASK_NAME;
	public final String DOWNLOAD_SERVERLIST_TASK_NAME;
	public final String SEARCH_TASK_NAME;

	// TABNAMES
	public final static String EDITSETTINGS_TABNAME = "Settings";
	public final static String SEARCH_TABNAME = "Playersearch";
	public final static String FAVOURITES_TABNAME = "Favourites";
	public final static String BLACKLIST_TABNAME = "Outsiders";
	public final static String MAIN_TABNAME = "Main";
	public final static String LASTCONNECTED_TABNAME = "History";

	// a short term for this game
	protected String gamePrefix = null;
	// this game's settings object
	protected SearchtoolSettings settings = null;
	// this game's config directory
	protected File gameConfigDirectory;
	// the file that stores the rules the user selected to be highlighted
	protected File rulesConfigFile;
	// the file that stores the button settings of the main list
	protected File mainListButtonSettingsFile;
	// the file that stores  the button settings of the favourite list
	protected File favouriteListButtonSettingsFile;
	// the file that stores the button settings of the black list
	protected File blackListButtonSettingsFile;
	// the file that stores the button settings of the history list
	protected File lastconnectedListButtonSettingsFile;
	// the file that stores  the button settings of the result list
	protected File resultListButtonSettingsFile;
	// the file that stores the complete list of servers
	protected File serverFile;
	// the file that stores the halflife settings
	protected File configFile;
	// the element controller for this game
	protected ElementController elementController = null;

	// components

	// the panel with the search players dialog
	protected SearchPlayersPanel searchPlayersPanel;
	// the panel with the edit settings dialog
	protected JPanel editSettingsPanel;
	// all tablemodels that provide a fast access to the data of the listmanagers
	protected SearchtoolTableModel blacklistTablemodel, favouritelistTablemodel, mainlistTablemodel, lastconnectedlistTablemodel;
	// this tablemodel contains slightly different data
	protected SearchtoolResultTableModel resultlistTablemodel;
	// the tables that display the data
	protected SearchtoolTable blacklistTable, favouritelistTable, hl_table, lastconnectedTable, searchresultlistTable;
	// the button trackers that keep track of selected buttons and store them on disk.
	protected ButtonTracker mainlistButtontracker, favouritelistButtontracker, blacklistButtontracker, lastconnectedlistButtontracker, resultlistButtontracker;
	// the listmanagers that handle (sort/filter) the data but do not provide fast access ( --> tablemodel)
	protected ServerListManager serverListManager, blacklistManager, favouritelistManager, lastconnectedlistManager;
	protected SearchtoolResultListManager resultlistManager;
	// the parser callback to parse stored data
	protected ServerElement.ListDataElementParserCallback xmlParserCallback = null;
	// the xmlwriter that stores the data in xml format to disk
	protected ServerElement.ListDataElementXMLWriter xmlWriter = null;
	// the buttonbars
	protected SearchtoolButtonBar mainlistButtonbar, favouritelistButtonbar, outsiderlistButtonbar, lastconnectedlistButtonbar, searchplayerlistButtonbar;
	// the searchtool tabs
	protected SearchtoolTab mainlistTab, favouritelistTab, blacklistTab, lastconnectedlistTab, searchplayerTab;
	// the one-and-only GUI
	protected GUI gui = null;
	// the table header listeners that take care of sorting and stuff
	protected SearchtoolTableHeaderSortingController lastconnectedSortingController, mainHeaderListener, favouriteHeaderListener, outsiderHeaderListener, resultHeaderListener;

	// the button controller that makes sure to select certain buttons simultaneously
	protected SynchronButtonController buttonController = new SynchronButtonController();

	// default actions
	protected ActionListener addServer;
	protected ActionListener makeFavourite;
	protected ActionListener makeOutsider;
	protected ActionListener getServerlist;
	protected ActionListener editSettings;
	protected ActionListener filterServerList;
	protected ActionListener getDetails;
	protected ActionListener getHelp;
	protected ActionListener quit;
	protected ActionListener searchServer;
	protected ActionListener masterFilter;
	protected ActionListener sortServerList;
	protected ActionListener pingServerList;
	protected ActionListener connect;

	// threads used to wrap table models
	protected Thread tablemodel_thread;
	protected Thread favouriteTableModelThread;
	protected Thread mainTableModelThread;
	protected Thread resultTableModelThread;
	protected Thread lastconnectedTableModelThread;

	protected GameController(String gamePrefix) {

		PING_TASK_NAME = gamePrefix + "_PING_TASK";
		SEARCH_TASK_NAME = gamePrefix + "_SEARCH_TASK";
		DOWNLOAD_SERVERLIST_TASK_NAME = gamePrefix + "_DOWNLOAD_TASK";

		this.gamePrefix = gamePrefix;
		String configDirectory = Searchtool.Constants.configDirectory;

		gameConfigDirectory = new File(configDirectory + File.separator + gamePrefix);
		mainListButtonSettingsFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.mainListButtonFileName);
		favouriteListButtonSettingsFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.favouriteListButtonFileName);
		blackListButtonSettingsFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.blacklistButtonFileName);
		lastconnectedListButtonSettingsFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.lastConnectedListButtonFileName);
		resultListButtonSettingsFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.resultListButtonFileName);
		rulesConfigFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.rulesConfigFileName);
		serverFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.serverFileName);
		configFile = new File(configDirectory + File.separator + gamePrefix + File.separator + this.configFileName);

		settings = new SearchtoolSettings(configFile);
		settings.load();
		elementController = new ElementController(this, serverFile);

		// components
		editSettingsPanel = new SearchtoolSettingsPanel(settings, "searchtool/files/" + gamePrefix + "_settings.html");
		searchPlayersPanel = new SearchPlayersPanel( this, new GamespyPlayers(), new GamespyDetails() );
	}

	/**
	 *  get the directory that contains this game's config files
	 */
	public File getGameConfigDir() {
		return gameConfigDirectory;
	}

	public void destroy() {
		gamePrefix = null;
		settings = null;
		gameConfigDirectory = null;
		rulesConfigFile = null;
		mainListButtonSettingsFile = null;
		favouriteListButtonSettingsFile = null;
		blackListButtonSettingsFile = null;
		lastconnectedListButtonSettingsFile = null;
		resultListButtonSettingsFile = null;
		serverFile = null;
		configFile = null;
		searchPlayersPanel = null;
		editSettingsPanel = null;
		blacklistTablemodel.destroy();
		blacklistTablemodel = null;
		favouritelistTablemodel.destroy();
		favouritelistTablemodel = null;
		mainlistTablemodel.destroy();
		mainlistTablemodel = null;
		lastconnectedlistTablemodel.destroy();
		lastconnectedlistTablemodel = null;
		resultlistTablemodel.destroy();
		resultlistTablemodel = null;
		elementController.destroy();
		elementController = null;
		blacklistTable = null;
		favouritelistTable = null;
		hl_table = null;
		lastconnectedTable = null;
		searchresultlistTable = null;
		mainlistButtontracker = null;
		favouritelistButtontracker = null;
		blacklistButtontracker = null;
		lastconnectedlistButtontracker = null;
		resultlistButtontracker = null;
		serverListManager.destroy();
		serverListManager = null;
		blacklistManager.destroy();
		blacklistManager = null;
		favouritelistManager.destroy();
		favouritelistManager = null;
		lastconnectedlistManager.destroy();
		lastconnectedlistManager = null;
		xmlParserCallback = null;
		xmlWriter = null;
		mainlistButtonbar = null;
		favouritelistButtonbar = null;
		outsiderlistButtonbar = null;
		lastconnectedlistButtonbar = null;
		searchplayerlistButtonbar = null;
		mainlistTab = null;
		favouritelistTab = null;
		blacklistTab = null;
		lastconnectedlistTab = null;
		searchplayerTab = null;
		gui = null;
		lastconnectedSortingController = null;
		mainHeaderListener = null;
		favouriteHeaderListener = null;
		outsiderHeaderListener = null;
		resultHeaderListener = null;
		buttonController = null;
		addServer = null;
		makeFavourite = null;
		makeOutsider = null;
		getServerlist = null;
		editSettings = null;
		filterServerList = null;
		getDetails = null;
		getHelp = null;
		quit = null;
		searchServer = null;
		masterFilter = null;
		sortServerList = null;
		pingServerList = null;
		connect = null;
		tablemodel_thread = null;
		favouriteTableModelThread = null;
		mainTableModelThread = null;
		resultTableModelThread = null;
		lastconnectedTableModelThread = null;
	}

	/**
	* save all data for a secure exit.
	*/
	public void safeExit() {
		// save serverlist
		elementController.saveServerList();
		// store buttonsettings
		mainlistButtontracker.save();
		blacklistButtontracker.save();
		favouritelistButtontracker.save();
		lastconnectedlistButtontracker.save();
		resultlistButtontracker.save();
		// store normal settings
		settings.save();
		// make garbageman ready
		destroy();
		// get garbageman
		System.gc();
	}

	/**
	 *  this method starts loading the serverlist from file.
	 */
	public void load(JProgressBar progressBar) {
		if (progressBar != null)
			progressBar.setString("Loading " + gamePrefix + " serverlist");
		elementController.loadServerList(progressBar);
	}

	/**
	 *  get a XMLWriter. used to save the serverlist
	 */
	public ServerElement.ListDataElementXMLWriter getXMLWriter() {
		return new ServerElement.ListDataElementXMLWriter(serverFile);
	}

	/**
	 *  get the XMLParser. used to parse the serverlist
	 */
	public ServerElement.ListDataElementParserCallback getXMLParser() {
		return new ServerElement.ListDataElementParserCallback();
	}

	/**
	 *  within this method all necessary objects should be created.
	 */

	protected void initBlacklistTable() {
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// BLACKLIST TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		blacklistButtontracker = new ButtonTracker(blackListButtonSettingsFile);
		blacklistManager = new ServerListManager(settings, blacklistButtontracker);
		// register manager
		elementController.registerListManager(blacklistManager, ElementController.BLACK_LIST);
		blacklistTablemodel = new SearchtoolTableModel(blacklistManager);
		blacklistTable = new SearchtoolTable(blacklistTablemodel);

		// wrap tablemodel in thread to make sure it gets updated
		tablemodel_thread = new Thread(blacklistTablemodel);
		tablemodel_thread.start();

		outsiderHeaderListener = new SearchtoolTableHeaderSortingController(blacklistTablemodel, blacklistButtontracker);
		blacklistTable.getTableHeader().addMouseListener(outsiderHeaderListener);
		outsiderHeaderListener.registerTable(blacklistTable);

		// add cell renderers
		String columname = blacklistTable.getColumnName(SearchtoolTableModel.RESPONSE);
		blacklistTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
		columname = blacklistTable.getColumnName(SearchtoolTableModel.PASSWORD);
		blacklistTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
	}

	protected void initFavouritelistTable() {
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// FAVOURITELIST TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		favouritelistButtontracker = new ButtonTracker(favouriteListButtonSettingsFile);
		favouritelistManager = new ServerListManager(settings, favouritelistButtontracker);
		// register listmanager
		elementController.registerListManager(favouritelistManager, ElementController.FAVOURITE_LIST);
		favouritelistTablemodel = new SearchtoolTableModel(favouritelistManager);
		favouritelistTable = new SearchtoolTable(favouritelistTablemodel);

		// wrap tablemodel in thread to make sure it gets updated
		favouriteTableModelThread = new Thread(favouritelistTablemodel);
		favouriteTableModelThread.start();

		favouriteHeaderListener = new SearchtoolTableHeaderSortingController(favouritelistTablemodel, favouritelistButtontracker);
		favouritelistTable.getTableHeader().addMouseListener(favouriteHeaderListener);
		favouriteHeaderListener.registerTable(favouritelistTable);

		// add cell renderers
		String columname = favouritelistTable.getColumnName(SearchtoolTableModel.RESPONSE);
		favouritelistTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
		columname = favouritelistTable.getColumnName(SearchtoolTableModel.PASSWORD);
		favouritelistTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
	}

	protected void initMainlistTable() {
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// MAIN TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		mainlistButtontracker = new ButtonTracker(mainListButtonSettingsFile);
		serverListManager = new ServerListManager(settings, mainlistButtontracker);
		// register listmanager
		elementController.registerListManager(serverListManager, ElementController.MAIN_LIST);
		mainlistTablemodel = new SearchtoolTableModel(serverListManager);
		// wrap tablemodel in thread to make sure it gets updated
		mainTableModelThread = new Thread(mainlistTablemodel);
		mainTableModelThread.start();
		// create table
		hl_table = new SearchtoolTable(mainlistTablemodel);

		mainHeaderListener = new SearchtoolTableHeaderSortingController(mainlistTablemodel, mainlistButtontracker);
		hl_table.getTableHeader().addMouseListener(mainHeaderListener);
		mainHeaderListener.registerTable(hl_table);

		// add cell renderers
		String columname = hl_table.getColumnName(SearchtoolTableModel.RESPONSE);
		hl_table.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
		columname = hl_table.getColumnName(SearchtoolTableModel.PASSWORD);
		hl_table.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
	}

	protected void initSearchresultlistTable() {
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// SEARCHRESULT TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		resultlistButtontracker = new ButtonTracker(resultListButtonSettingsFile);
		// initialize the customtablemodle
		resultlistManager = new SearchtoolResultListManager(settings, resultlistButtontracker);
		resultlistTablemodel = new SearchtoolResultTableModel(resultlistManager);
		searchresultlistTable = new SearchtoolTable(resultlistTablemodel);

		resultTableModelThread = new Thread(resultlistTablemodel);
		resultTableModelThread.start();

		resultHeaderListener = new SearchtoolTableHeaderSortingController(resultlistTablemodel, resultlistButtontracker);
		searchresultlistTable.getTableHeader().addMouseListener(resultHeaderListener);
		resultHeaderListener.registerTable(searchresultlistTable);

		// custom renderers
		String columname = resultlistTablemodel.getColumnName(SearchtoolTableModel.PASSWORD);
		searchresultlistTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
		// customize columsize
		searchresultlistTable.getColumn("PLAYER").setPreferredWidth(200);
	}

	protected void initLastconnectedlistTable() {
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// LASTCONNECTED TABLE
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		lastconnectedlistButtontracker = new ButtonTracker(lastconnectedListButtonSettingsFile);
		// initialize the customtablemodel
		lastconnectedlistManager = new ServerListManager(settings, lastconnectedlistButtontracker);
		// register listmanager
		elementController.registerListManager(lastconnectedlistManager, ElementController.CONNECTED_LIST);
		lastconnectedlistTablemodel = new SearchtoolLastconnectedTableModel(lastconnectedlistManager);
		lastconnectedTable = new SearchtoolTable(lastconnectedlistTablemodel);

		lastconnectedTableModelThread = new Thread(lastconnectedlistTablemodel); // wrap model in thread to take care of
		lastconnectedTableModelThread.start(); // the updating of the table	

		lastconnectedSortingController = new SearchtoolTableHeaderSortingController(lastconnectedlistTablemodel, lastconnectedlistButtontracker);
		lastconnectedTable.getTableHeader().addMouseListener(lastconnectedSortingController);
		lastconnectedSortingController.registerTable(lastconnectedTable);

		// add cell renderers
		String columname = lastconnectedlistTablemodel.getColumnName(SearchtoolTableModel.RESPONSE);
		lastconnectedTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());
		columname = lastconnectedlistTablemodel.getColumnName(SearchtoolTableModel.PASSWORD);
		lastconnectedTable.getColumn(columname).setCellRenderer(new SearchtoolTableCellRenderer());

		// customize columsize
		lastconnectedTable.getColumn("CONNECTIONDATE").setPreferredWidth(200);
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// INIT TABS
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	protected void initActionListeners() {
		// default actions
		addServer = new AddServerToListActionListener(elementController, this, new HlServerElement());
		makeFavourite = new AddToFavouritesActionListener(elementController);
		makeOutsider = new AddToOutsidersActionListener(elementController);
		getServerlist = new DownloadServersActionListener(elementController, this, buttonController, new GamespyServerRequest(settings, gamePrefix));
		editSettings = new EditSettingsActionListener(editSettingsPanel, EDITSETTINGS_TABNAME, buttonController);
		filterServerList = new FilterServerlistActionListener();
		getDetails = new GetDetailsActionListener(new GamespyPing(), new GamespyRules(), new GamespyDetails(), new GamespyPlayers(), rulesConfigFile, settings);
		getHelp = new HelpActionListener("searchtool/files/" + gamePrefix + "_help.html");
		quit = new QuitActionListener();
		searchServer = new SearchServerActionListener(elementController);
		masterFilter = new SetMasterserverFilterActionListener(settings, buttonController);
		sortServerList = new SortServerlistActionListener();
		pingServerList = new PingServersActionListener(this, settings, new GamespyPing(), new GamespyDetails(), new GamespyRules());
		connect = new ConnectActionListener(new GamespyDetails(), settings);
	}

	public void initComponents() {

		// actions
		initActionListeners();

		// INIT TABLES
		initBlacklistTable();
		initFavouritelistTable();
		initMainlistTable();
		initSearchresultlistTable();
		initLastconnectedlistTable();

		// buttonbars
		initMainButtonBar();
		initFavouriteButtonBar();
		initOutsiderButtonBar();
		initSearchButtonBar();
		initLastconnectedButtonBar();

		// create and add tabs
		gui = Searchtool.getGui();
		GameFrontend thisFrontend = gui.setupNewGameFrontend(this.gamePrefix);
		mainlistTab = new SearchtoolTab(mainlistButtonbar, getMainTable(), SearchtoolTab.MAINLIST_TAB);
		mainlistTab.setName(MAIN_TABNAME);
		thisFrontend.addTab(mainlistTab);
		favouritelistTab = new SearchtoolTab(favouritelistButtonbar, getFavouritelistTable(), SearchtoolTab.FAVOURITE_TAB);
		favouritelistTab.setName(FAVOURITES_TABNAME);
		thisFrontend.addTab(favouritelistTab);
		blacklistTab = new SearchtoolTab(outsiderlistButtonbar, getBlacklistTable(), SearchtoolTab.BLACKLIST_TAB);
		blacklistTab.setName(BLACKLIST_TABNAME);
		thisFrontend.addTab(blacklistTab);
		lastconnectedlistTab = new SearchtoolTab(lastconnectedlistButtonbar, getLastconnectedListTable(), SearchtoolTab.CONNECTED_TAB);
		lastconnectedlistTab.setName(LASTCONNECTED_TABNAME);
		thisFrontend.addTab(lastconnectedlistTab);
		searchplayerTab = new SearchtoolTab(searchplayerlistButtonbar, searchPlayersPanel, searchresultlistTable, SearchtoolTab.PLAYERSEARCH_TAB);
		searchplayerTab.setName(SEARCH_TABNAME);
		thisFrontend.addTab(searchplayerTab);

		//set mainlist tab on top
		thisFrontend.selectTab(MAIN_TABNAME);

		// restore masterserver (buttons) settings
		int masterFilterEnabled = settings.getValueAsInt(SearchtoolSettings.MASTERLEVEL_FILTER);
		if (masterFilterEnabled == SearchtoolSettings.ENABLED)
			buttonController.selectButtonsOfType(true, buttonController.MASTERSERVER_FILTER);
	}

	protected void initMainButtonBar() {
		mainlistButtonbar = new SearchtoolButtonBar(buttonController);
		mainlistButtonbar.setActionListener(addServer, mainlistButtonbar.ADD_SERVER);
		mainlistButtonbar.setActionListener(makeFavourite, mainlistButtonbar.MAKE_FAVOURITE);
		mainlistButtonbar.setActionListener(makeOutsider, mainlistButtonbar.MAKE_OUTSIDER);
		mainlistButtonbar.setActionListener(getServerlist, mainlistButtonbar.GET_SERVERLIST);
		mainlistButtonbar.setActionListener(editSettings, mainlistButtonbar.EDIT_SETTINGS);
		mainlistButtonbar.setActionListener(filterServerList, mainlistButtonbar.LOCAL_FILTER);
		mainlistButtonbar.setActionListener(getDetails, mainlistButtonbar.DISPLAY_DETAILS);
		mainlistButtonbar.setActionListener(getHelp, mainlistButtonbar.HELP);
		mainlistButtonbar.setActionListener(quit, mainlistButtonbar.QUIT);
		mainlistButtonbar.setActionListener(searchServer, mainlistButtonbar.FIND_SERVER);
		mainlistButtonbar.setActionListener(masterFilter, mainlistButtonbar.MASTERSERVER_FILTER);
		mainlistButtonbar.setActionListener(sortServerList, mainlistButtonbar.SORT);
		mainlistButtonbar.setActionListener(pingServerList, mainlistButtonbar.PING);
		mainlistButtonbar.setActionListener(connect, mainlistButtonbar.CONNECT);
		// before init add additional filter panel
		mainlistButtonbar.setAdditionalPanel(new FilterSettingsPanel(settings));
		mainlistButtonbar.initToolBar();
		// register buttonbar on tablemodel to update buttons depending on user selections
		mainlistTablemodel.registerButtonBar(mainlistButtonbar);
		mainlistTablemodel.updateButtons();
	}

	protected void initFavouriteButtonBar() {
		favouritelistButtonbar = new SearchtoolButtonBar(buttonController);
		favouritelistButtonbar.setActionListener(addServer, mainlistButtonbar.ADD_SERVER);
		favouritelistButtonbar.setActionListener(makeFavourite, mainlistButtonbar.MAKE_FAVOURITE);
		favouritelistButtonbar.setActionListener(makeOutsider, mainlistButtonbar.MAKE_OUTSIDER);
		favouritelistButtonbar.setActionListener(getServerlist, mainlistButtonbar.GET_SERVERLIST);
		favouritelistButtonbar.setActionListener(editSettings, mainlistButtonbar.EDIT_SETTINGS);
		favouritelistButtonbar.setActionListener(filterServerList, mainlistButtonbar.LOCAL_FILTER);
		favouritelistButtonbar.setActionListener(getDetails, mainlistButtonbar.DISPLAY_DETAILS);
		favouritelistButtonbar.setActionListener(getHelp, mainlistButtonbar.HELP);
		favouritelistButtonbar.setActionListener(quit, mainlistButtonbar.QUIT);
		favouritelistButtonbar.setActionListener(searchServer, mainlistButtonbar.FIND_SERVER);
		favouritelistButtonbar.setActionListener(masterFilter, mainlistButtonbar.MASTERSERVER_FILTER);
		favouritelistButtonbar.setActionListener(sortServerList, mainlistButtonbar.SORT);
		favouritelistButtonbar.setActionListener(pingServerList, mainlistButtonbar.PING);
		favouritelistButtonbar.setActionListener(connect, mainlistButtonbar.CONNECT);
		// before init add additional filter panel
		favouritelistButtonbar.setAdditionalPanel(new FilterSettingsPanel(settings));
		favouritelistButtonbar.initToolBar();
		// register buttonbar on tablemodel to update buttons depending on user selections
		favouritelistTablemodel.registerButtonBar(favouritelistButtonbar);
		favouritelistTablemodel.updateButtons();
	}

	protected void initOutsiderButtonBar() {
		outsiderlistButtonbar = new SearchtoolButtonBar(buttonController);
		outsiderlistButtonbar.setActionListener(addServer, mainlistButtonbar.ADD_SERVER);
		outsiderlistButtonbar.setActionListener(makeFavourite, mainlistButtonbar.MAKE_FAVOURITE);
		outsiderlistButtonbar.setActionListener(makeOutsider, mainlistButtonbar.MAKE_OUTSIDER);
		outsiderlistButtonbar.setActionListener(getServerlist, mainlistButtonbar.GET_SERVERLIST);
		outsiderlistButtonbar.setActionListener(editSettings, mainlistButtonbar.EDIT_SETTINGS);
		outsiderlistButtonbar.setActionListener(filterServerList, mainlistButtonbar.LOCAL_FILTER);
		outsiderlistButtonbar.setActionListener(getDetails, mainlistButtonbar.DISPLAY_DETAILS);
		outsiderlistButtonbar.setActionListener(getHelp, mainlistButtonbar.HELP);
		outsiderlistButtonbar.setActionListener(quit, mainlistButtonbar.QUIT);
		outsiderlistButtonbar.setActionListener(searchServer, mainlistButtonbar.FIND_SERVER);
		outsiderlistButtonbar.setActionListener(masterFilter, mainlistButtonbar.MASTERSERVER_FILTER);
		outsiderlistButtonbar.setActionListener(sortServerList, mainlistButtonbar.SORT);
		outsiderlistButtonbar.setActionListener(pingServerList, mainlistButtonbar.PING);
		outsiderlistButtonbar.setActionListener(connect, mainlistButtonbar.CONNECT);
		// before init add additional filter panel
		outsiderlistButtonbar.setAdditionalPanel(new FilterSettingsPanel(settings));
		outsiderlistButtonbar.initToolBar();
		// register buttonbar on tablemodel to update buttons depending on user selections
		blacklistTablemodel.registerButtonBar(outsiderlistButtonbar);
		blacklistTablemodel.updateButtons();
	}

	protected void initSearchButtonBar() {
		searchplayerlistButtonbar = new SearchtoolButtonBar(buttonController);
		searchplayerlistButtonbar.setActionListener(addServer, mainlistButtonbar.ADD_SERVER);
		searchplayerlistButtonbar.setActionListener(makeFavourite, mainlistButtonbar.MAKE_FAVOURITE);
		searchplayerlistButtonbar.setActionListener(makeOutsider, mainlistButtonbar.MAKE_OUTSIDER);
		searchplayerlistButtonbar.setActionListener(getServerlist, mainlistButtonbar.GET_SERVERLIST);
		searchplayerlistButtonbar.setActionListener(editSettings, mainlistButtonbar.EDIT_SETTINGS);
		searchplayerlistButtonbar.setActionListener(filterServerList, mainlistButtonbar.LOCAL_FILTER);
		searchplayerlistButtonbar.setActionListener(getDetails, mainlistButtonbar.DISPLAY_DETAILS);
		searchplayerlistButtonbar.setActionListener(getHelp, mainlistButtonbar.HELP);
		searchplayerlistButtonbar.setActionListener(quit, mainlistButtonbar.QUIT);
		searchplayerlistButtonbar.setActionListener(searchServer, mainlistButtonbar.FIND_SERVER);
		searchplayerlistButtonbar.setActionListener(masterFilter, mainlistButtonbar.MASTERSERVER_FILTER);
		searchplayerlistButtonbar.setActionListener(sortServerList, mainlistButtonbar.SORT);
		searchplayerlistButtonbar.setActionListener(pingServerList, mainlistButtonbar.PING);
		searchplayerlistButtonbar.setActionListener(connect, mainlistButtonbar.CONNECT);
		// before init add additional filter panel
		searchplayerlistButtonbar.setAdditionalPanel(new FilterSettingsPanel(settings));
		searchplayerlistButtonbar.initToolBar();
		// register buttonbar on tablemodel to update buttons depending on user selections
		resultlistTablemodel.registerButtonBar(searchplayerlistButtonbar);
		resultlistTablemodel.updateButtons();

		// disable unused buttons
		searchplayerlistButtonbar.getButton(SearchtoolButtonBar.FIND_SERVER).setEnabled(false);
		//hl_searchplayertab_buttonbar.getButton(SearchtoolButtonBar.SORT).setEnabled(false);
		//hl_searchplayertab_buttonbar.getButton(SearchtoolButtonBar.LOCAL_FILTER).setEnabled(false);
		searchplayerlistButtonbar.getButton(SearchtoolButtonBar.ADD_SERVER).setEnabled(false);
	}

	protected void initLastconnectedButtonBar() {
		lastconnectedlistButtonbar = new SearchtoolButtonBar(buttonController);
		lastconnectedlistButtonbar.setActionListener(addServer, mainlistButtonbar.ADD_SERVER);
		lastconnectedlistButtonbar.setActionListener(makeFavourite, mainlistButtonbar.MAKE_FAVOURITE);
		lastconnectedlistButtonbar.setActionListener(makeOutsider, mainlistButtonbar.MAKE_OUTSIDER);
		lastconnectedlistButtonbar.setActionListener(getServerlist, mainlistButtonbar.GET_SERVERLIST);
		lastconnectedlistButtonbar.setActionListener(editSettings, mainlistButtonbar.EDIT_SETTINGS);
		lastconnectedlistButtonbar.setActionListener(filterServerList, mainlistButtonbar.LOCAL_FILTER);
		lastconnectedlistButtonbar.setActionListener(getDetails, mainlistButtonbar.DISPLAY_DETAILS);
		lastconnectedlistButtonbar.setActionListener(getHelp, mainlistButtonbar.HELP);
		lastconnectedlistButtonbar.setActionListener(quit, mainlistButtonbar.QUIT);
		lastconnectedlistButtonbar.setActionListener(searchServer, mainlistButtonbar.FIND_SERVER);
		lastconnectedlistButtonbar.setActionListener(masterFilter, mainlistButtonbar.MASTERSERVER_FILTER);
		lastconnectedlistButtonbar.setActionListener(sortServerList, mainlistButtonbar.SORT);
		lastconnectedlistButtonbar.setActionListener(pingServerList, mainlistButtonbar.PING);
		lastconnectedlistButtonbar.setActionListener(connect, mainlistButtonbar.CONNECT);

		// before init add additional filter panel
		lastconnectedlistButtonbar.setAdditionalPanel(new FilterSettingsPanel(settings));
		lastconnectedlistButtonbar.initToolBar();

		// register buttonbar on tablemodel to update buttons depending on user selections
		lastconnectedlistTablemodel.registerButtonBar(lastconnectedlistButtonbar);
		lastconnectedlistTablemodel.updateButtons();

		// disable unused buttons
		lastconnectedlistButtonbar.getButton(SearchtoolButtonBar.ADD_SERVER).setEnabled(false);
	}

	public SearchtoolTable getBlacklistTable() {
		return blacklistTable;
	}
	public SearchtoolTable getFavouritelistTable() {
		return favouritelistTable;
	}
	public SearchtoolTable getMainTable() {
		return hl_table;
	}
	public SearchtoolTable getLastconnectedListTable() {
		return lastconnectedTable;
	}
	public SearchtoolSettings getSearchtoolSettings() {
		return settings;
	}
	public ElementController getElementController() {
		return elementController;
	}
	public SearchtoolResultTableModel getResultTableModel() {
		return resultlistTablemodel;
	}
}
