/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

package searchtool;

import searchtool.components.*;
import searchtool.util.IconLib;
import searchtool.wrappers.*;
import searchtool.protocol.*;
import searchtool.interfaces.*;
import searchtool.util.*;
import searchtool.*;
import java.util.*;
import java.io.*;
import java.lang.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
// thirdparty
import gnu.trove.*;

/**
 *  This class holds some methods to be used for connecting to a gameserver.  Extending classes might
 *  want to override the <code>connect</code> method to be able to connect to gameservers.
 * 
 * This class also makes sure that it is possible to (try) to connect to more than one server at a time.
 * @author thorsten
 *
 */
public class SearchtoolConnect {

	private SearchtoolProtocol details = null;
	private SearchtoolSettings settings = null;
	private SearchtoolDialogs.ConnectDialog dialog = null;
	private boolean stopConnect = false;

	private static Vector connections = new Vector();

	/**
	 *  The default constructor.
	 *  @param details is a <code>SearchtoolProtcol</code> instance that is used to update the player information on the server.
	 */
	public SearchtoolConnect(SearchtoolProtocol details, SearchtoolSettings settings) {
		this.details = details;
		this.settings = settings;
	}

	/**
	 *  Store a connect to make it possible that it can be stopped lateron.
	 */
	public static void addConnect(SearchtoolConnect connect) {
		connections.add(connect);
	}

	/**
	 *  Tell all possible connections to stop imediately. This calls
	 *  the <code>stopConnect</code> method for all running connections.
	 * Synchronization is important because it might interfere with <code>removeConnect</code>
	 */
	public synchronized static void stopAllConnects() {
		for (int i = 0; i < connections.size(); i++) {
			((SearchtoolConnect) connections.get(i)).stopConnect();
		}
		connections.clear();
	}

	/**
	 *  Stop connecting by calling this method
	 */
	public void stopConnect() {
		stopConnect = true;
		dialog.closeDialog();
	}

	/**
	 *  Remove the given connect from the list of stored connects. (after cancel for instance) .
	 *  Synchronization is important because it might interfere with <code>stopAllConnects</code>
	 */
	public synchronized static void removeConnect(SearchtoolConnect connect) {
		for (int i = 0; i < connections.size(); i++) {
			if (((SearchtoolConnect) connections.get(i)).equals(connect)) {
				connections.remove(i);
				break;
			}
		}
	}

	/**
	 * Start the countdown that will finally shutdown searchtool
	 *  @return  this method will return (void) only if the countdown has been cancelled. otherwise it will shutdown searchtool properly.
	 */
	public void runCountDown() {
		final int countdown = Searchtool.getGlobalSettings().getValueAsInt(SearchtoolSettings.SHUTDOWN_TIMEOUT);
		final JOptionPane pane = new JOptionPane("Countdown started.  Push button to cancel shutdown of Searchtool.", JOptionPane.NO_OPTION);
		final JDialog dialog = pane.createDialog(Searchtool.getGui(), "countdown ...");

		// slow down some stuff.
		Searchtool.getTaskManager().stopAndRemoveAllTasks();
		Searchtool.sleep(true);

		// countdown
		new Thread(new Runnable() {
			public void run() {
				int counter = 0;
				while (true) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					counter += 1;
					if (countdown - counter <= 0) {
						Searchtool.safeExit();
						break;
					}
				}
			}
		}).start();
		dialog.show();

		// if we get here the countdown has been cancelled ---> weakup and return
		Searchtool.sleep(false);
		return;
	}

	/**
	 *  Tries to execute the given <code>String</code> as a command.
	 *  @param the server element to connect to.
	 */
	protected final void launch(ServerElement finalServerElement) {

		// slot free -->connect.  maybe start countdown.
		String command = settings.getValue(SearchtoolSettings.CONNECT_COMMAND) + " " + settings.getConnectPostfix(finalServerElement);
		try {
			Runtime runtime = Runtime.getRuntime();
			Process process = runtime.exec(command);
		} catch (Exception e) {
			stopConnect();
			removeConnect(SearchtoolConnect.this);
			SearchtoolDialogs.ErrorDialog errorDialog = new SearchtoolDialogs.ErrorDialog("could not execute:\n" + command);
			errorDialog.showErrorDialog();
			return;
		}
		// LAST STEPS
		finalServerElement.setDate(new Date());
		// add to history
		finalServerElement.setLastConnectedlist(true);
		
		System.out.println("/////////////////////////////////////////////////////////////////////////////////////");
		System.out.println(command);
		System.out.println("/////////////////////////////////////////////////////////////////////////////////////");
		
		if (Searchtool.getGlobalSettings().getValueAsInt(SearchtoolSettings.SHUTDOWN_TIMEOUT) >= 0) {
			runCountDown();
		} else {
			Searchtool.safeExit();
		}
	}

	/**
	 *  this method contains the logic to wait for a free slot.  
	 *  when it is time to connect it uses the <code>launch</code> method.
	 *  NOTE: this method does NOT block.  it just starts a new thread.
	 *  @param serverElement represents the server to connect to.
	 */
	public void connect(ServerElement serverElement) {
		final ServerElement finalServerElement = serverElement;
		new Thread() {
			public void run() {

				dialog = new SearchtoolDialogs.ConnectDialog();
				dialog.showDialog();

				// arm buttons
				dialog.setActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						stopConnect();
						removeConnect(SearchtoolConnect.this);
					}
				}, SearchtoolDialogs.ConnectDialog.CANCEL_BUTTON);

				dialog.setActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						launch(finalServerElement);
					}
				}, SearchtoolDialogs.ConnectDialog.CONNECT_BUTTON);

				// update playerstatus (in case of no success , retry once)
				details.setServerElement(finalServerElement);
				 if ( ! details.update() )
				 	details.update();

				int retryCount = 0;
				while (finalServerElement.getMaxplayers().intValue() - finalServerElement.getCurrentplayers().intValue() <= 0) {
					try {
						Thread.sleep(800);
					} catch (Exception e) {
						e.printStackTrace();
					}
					retryCount++;

					// update
					details.update();
					String status =
						"  Server: "
							+ finalServerElement.getServername()
							+ "\n"
							+ "  Currentplayers: "
							+ finalServerElement.getCurrentplayers()
							+ "\n"
							+ "  Maxplayers: "
							+ finalServerElement.getMaxplayers()
							+ "\n\n"
							+ "  Retry Nr.: "
							+ retryCount;
					dialog.setText(status);
					if (stopConnect) {
						removeConnect(SearchtoolConnect.this);
						return;
					}
				}
				launch(finalServerElement);
			}
		}
		.start();
	}
}
