/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package searchtool;

import searchtool.wrappers.*;
import searchtool.interfaces.*;
import searchtool.components.*;
import searchtool.actions.*;
import searchtool.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;

/**
 * @author thorsten
 *
 * 
 */
public abstract class SelectedElementTracker extends AbstractTableModel implements Runnable {

	public static final int RESPONSE = 0, SERVERNAME = 1, IP = 2, PING = 3, MAP = 4, CURRENTPLAYERS = 5, MAXPLAYERS = 6, PASSWORD = 7;

	private ElementListManager msl;
	private int columncount;
	// the selected element from the jtable.
	protected Element selectedElement;
	// used in getValueAt()
	private Element lde;
	// a copy of the complete serverlist
	protected Element[] completeList;
	// a copy of the filteredList
	protected Element[] filteredList;
	// the corresponding buttonbar if registered.
	protected SearchtoolButtonBar buttonbar = null;
	// the buttontracker
	protected ButtonTracker buttonTracker = null;
	// the corresponding table, if registered
	private SearchtoolTable searchtoolTable = null;
	// a flag that indicates that this thread (runnable) should keep going ( or not )
	private boolean keepGoing = true;

	public SelectedElementTracker(ElementListManager MSL) {
		msl = MSL;
		columncount = 8;
		bufferNewLists();
		setSelectedElement(null);
		buttonTracker = msl.getButtonTracker();
	}
	
	public void destroy(){
		keepGoing = false;
		/*
		msl = null;
		selectedElement = null;
		lde = null;
		completeList = null;
		filteredList = null;
		buttonbar = null;
		buttonTracker = null;
		searchtoolTable = null;	
		*/
	}

	public ButtonTracker getButtonTracker() {
		return buttonTracker;
	}

	public ElementListManager getListManager() {
		return msl;
	}

	public void registerButtonBar(SearchtoolButtonBar buttonbar) {
		this.buttonbar = buttonbar;
	}

	public synchronized void setSelectedElement(Element ELEMENT) {
		setSelectedElement(getIndexOf(ELEMENT));
	}

	public void registerTable(SearchtoolTable searchtoolTable) {
		this.searchtoolTable = searchtoolTable;
	}

	public synchronized void setSelectedElement(int INDEX) {
		if (INDEX < 0) {
			updateButtons();
			selectedElement = null;
			return;
		}
		if (buttonTracker != null && buttonTracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
			selectedElement = filteredList[INDEX];
			updateButtons();
			return;
		}
		selectedElement = completeList[INDEX];
		updateButtons();
		return;
	}

	public synchronized int getSelectedIndex() {
		if (selectedElement == null)
			return -1;
		return getIndexOf(selectedElement);
	}

	public Element getSelectedElement() {
		return selectedElement;
	}

	public synchronized void addElement(Element element) {
		if (msl != null)
			msl.addElement(element);
	}

	public abstract void updateButtons();

	/**
	 * buffer a copy of the lists here to improve performance because of less method
	 * calls and avoiding the synchronized methods in the listmanager.
	 */
	public void bufferNewLists() {
		Element[] tmp = msl.getCompleteList();
		Element[] tmpServerElement = new Element[tmp.length];
		System.arraycopy(tmp, 0, tmpServerElement, 0, tmp.length);
		completeList = tmpServerElement;
		tmp = msl.getFilteredList();
		tmpServerElement = new Element[tmp.length];
		System.arraycopy(tmp, 0, tmpServerElement, 0, tmp.length);
		filteredList = tmpServerElement;
		if (selectedElement != null && msl.isElementInList(selectedElement) == false) {
			setSelectedElement(null);
		}
	}

	/**
	 * return element of choice from either the main serverlist, or
	 * the filtered serverlist. (depending on current shown list)
	 */
	public synchronized Element getElement(int INDEX) {
		// filtered list
		if (buttonTracker != null && buttonTracker.filterEnabled()) {
			if (filteredList.length > INDEX)
				return filteredList[INDEX];
			else
				return null;
		}
		// complete list
		if (completeList.length > INDEX)
			return completeList[INDEX];
		else
			return null;
	}

	/**
	 *  return the index of the given element in the currently selected list of this tablemodel.
	 *  returns value < 0 if element not found.  
	 *  warning:  make sure to check if the index is within bounds when using it !!!
	 */
	public synchronized int getIndexOf(Element LDE) {
		if (LDE == null)
			return -1;
		// get a reference on the proper list in case 
		// some other thread updates the lists.
		Element[] processedList = null;
		// if filtered
		if (buttonTracker != null && buttonTracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
			processedList = filteredList;
			for (int i = 0; i < processedList.length; i++) {
				if (processedList[i].compareTo(LDE) == 0) {
					return i;
				}
			}
			return -1;
		}
		// not filtered
		processedList = completeList;
		for (int i = 0; i < processedList.length; i++) {
			if (processedList[i].compareTo(LDE) == 0) {
				return i;
			}
		}
		return -1;
	}

	public abstract int getColumnCount();

	public int getRowCount() {
		if (buttonTracker != null && buttonTracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
			return filteredList.length;
		}
		return completeList.length;
	}

	public abstract Object getValueAt(int row, int column);

	public abstract String getColumnName(int column);

	/*
	 * interface method.
	 * this runs sorting and filter tasks.
	 */

	public void run() {
		while (keepGoing) {
			// obey filter interval
			try {
				Thread.sleep(Searchtool.getGlobalSettings().getValueAsInt(SearchtoolSettings.SORTANDFILTER_INTERVAL));
			} catch (Exception e) {
				;
			}
			// sleep loooooooong time if ordered
			while (Searchtool.getGlobalSettings().getValueAsInt(SearchtoolSettings.SLEEPING) == SearchtoolSettings.ENABLED) {
				try {
					Thread.sleep(1000);
				} catch (Exception e) {
					;
				}
			}

			// sort if button-tracker settings permit
			if (buttonTracker != null && buttonTracker.getValue(ButtonTracker.SORT_BUTTON).equals(ButtonTracker.ENABLED)) {
				msl.sortElementList();
			}

			try {
				Thread.yield();
			} catch (Exception e) {
				;
			}
			// filter if button-tracker settings say enabled
			if (buttonTracker != null && buttonTracker.getValue(ButtonTracker.LOCAL_FILTER_BUTTON).equals(ButtonTracker.ENABLED)) {
				msl.filterElementList();
			}

			try {
				Thread.yield();
			} catch (Exception e) {
				;
			}

			bufferNewLists();

			// update table
			if (searchtoolTable != null) {
				searchtoolTable.selectElement();
			}

			updateButtons();
			fireTableDataChanged();

		}
	}
}
