/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * SearchtoolButtonBar.java
 *
 * Created on March 11, 2002, 12:51 PM
 */

package searchtool.components;

import searchtool.util.*;
import searchtool.*;
import searchtool.halflife.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import searchtool.actions.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.util.*;

/**
 * this class is the toolbar that holds all the buttons.
 * the buttons can be split up into the following sections:
 *
 * WORLD-BUTTONS:
 * these buttons do not affect a single list, but
 * all of them.
 * 
 * - download serverlist action
 * - (master)filter serverlist
 *
 *
 * TASK-ON-SINGLE-LIST:
 * these buttons perform tasks on the servers of one
 * single list.
 * - ping servers
 * - sort serverlist
 * - (local)filter serverlist
 * - find server
 * - add server
 *
 * ACTION-ON-SINGLE-SERVER:
 * these buttons change the state of a single (selected)
 * server.
 * - make favourite
 * - make outsider
 * - give detailed information
 * - connect
 *
 * NO-CATEGORY:
 * - edit settings
 * - help
 * - exit
 *
 */
public class SearchtoolButtonBar extends JToolBar {

	public static final int GET_SERVERLIST = 0,
		MASTERSERVER_FILTER = 1,
		SEARCH_PLAYERS = 2,
		EDIT_SETTINGS = 3,
		QUIT = 4,
		SORT = 5,
		MAKE_FAVOURITE = 6,
		MAKE_OUTSIDER = 7,
		CONNECT = 8,
		DISPLAY_DETAILS = 9,
		LOCAL_FILTER = 10,
		CLOSE = 11,
		HELP = 12,
		FIND_SERVER = 13,
		ADD_SERVER = 14,
		PING = 15;

	private JToggleButton[] buttons = new JToggleButton[16];
	private ActionListener[] listeners = new ActionListener[16];
	private JPanel additionalPanel = null;
	private SynchronButtonController buttonController = null;

	/** Creates new SearchtoolButtonBar */
	public SearchtoolButtonBar(SynchronButtonController buttonController) {
		this.buttonController = buttonController;
		// config toolbar
		setFloatable(false);
		setOrientation(JToolBar.HORIZONTAL);
	}

	public void initToolBar() {
		// add buttons and separators to this toolbar
		addSeparator();
		if (listeners[GET_SERVERLIST] != null) {
			buttons[GET_SERVERLIST] = (JToggleButton) add(createButton(IconLib.getGetServerlistIcon(), listeners[GET_SERVERLIST]));
			if (buttonController != null) {
				buttonController.registerButtonOfType(buttons[GET_SERVERLIST], buttonController.DOWNLOAD_SERVERLIST);
			}
			buttons[GET_SERVERLIST].setToolTipText("Start/Stop download new serverlist");
		}
		if (listeners[MASTERSERVER_FILTER] != null) {
			buttons[MASTERSERVER_FILTER] = (JToggleButton) add(createButton(IconLib.getMasterserverFilterIcon(), listeners[MASTERSERVER_FILTER]));
			if (buttonController != null) {
				buttonController.registerButtonOfType(buttons[MASTERSERVER_FILTER], buttonController.MASTERSERVER_FILTER);
			}
			buttons[MASTERSERVER_FILTER].setToolTipText("Switch masterserver filter on/off");
		}
		if (listeners[SEARCH_PLAYERS] != null) {
			buttons[SEARCH_PLAYERS] = (JToggleButton) add(createButton(IconLib.getSearchPlayersIcon(), listeners[SEARCH_PLAYERS]));
			if (buttonController != null) {
				buttonController.registerButtonOfType(buttons[SEARCH_PLAYERS], buttonController.SEARCH_PLAYERS);
			}
			buttons[SEARCH_PLAYERS].setToolTipText("Open/Close search-players tab");
		}

		addSeparator();

		if (listeners[PING] != null) {
			buttons[PING] = (JToggleButton) add(createButton(IconLib.getPingIcon(), listeners[PING]));
			buttons[PING].setToolTipText("Start/Stop ping on all servers in this list");
		}
		if (listeners[SORT] != null) {
			buttons[SORT] = (JToggleButton) add(createButton(IconLib.getSortIcon(), listeners[SORT]));
			buttons[SORT].setToolTipText("Switch continuous sorting of this serverlist on/off");
		}
		if (listeners[LOCAL_FILTER] != null) {
			buttons[LOCAL_FILTER] = (JToggleButton) add(createButton(IconLib.getLocalFilterIcon(), listeners[LOCAL_FILTER]));
			buttons[LOCAL_FILTER].setToolTipText("Switch local filter on/off");
		}
		if (listeners[FIND_SERVER] != null) {
			buttons[FIND_SERVER] = (JToggleButton) add(createButton(IconLib.getFindServerIcon(), listeners[FIND_SERVER]));
			buttons[FIND_SERVER].setToolTipText("Find server in list");
		}
		if (listeners[ADD_SERVER] != null) {
			buttons[ADD_SERVER] = (JToggleButton) add(createButton(IconLib.getAddServerIcon(), listeners[ADD_SERVER]));
			buttons[ADD_SERVER].setToolTipText("Add new Server to list");
		}

		addSeparator();

		if (listeners[MAKE_FAVOURITE] != null) {
			buttons[MAKE_FAVOURITE] = (JToggleButton) add(createButton(IconLib.getMakeFavouriteIcon(), listeners[MAKE_FAVOURITE]));
			buttons[MAKE_FAVOURITE].setToolTipText("Add selected server to favourites");
		}
		if (listeners[MAKE_OUTSIDER] != null) {
			buttons[MAKE_OUTSIDER] = (JToggleButton) add(createButton(IconLib.getMakeOutsiderIcon(), listeners[MAKE_OUTSIDER]));
			buttons[MAKE_OUTSIDER].setToolTipText("Add selected server to outsiders");
		}
		if (listeners[CONNECT] != null) {
			buttons[CONNECT] = (JToggleButton) add(createButton(IconLib.getConnectIcon(), listeners[CONNECT]));
			buttons[CONNECT].setToolTipText("Connect to selected server");
		}
		if (listeners[DISPLAY_DETAILS] != null) {
			buttons[DISPLAY_DETAILS] = (JToggleButton) add(createButton(IconLib.getDisplayDetailsIcon(), listeners[DISPLAY_DETAILS]));
			buttons[DISPLAY_DETAILS].setToolTipText("Display details on selected server");
		}

		addSeparator();
		addSeparator();

		if (listeners[EDIT_SETTINGS] != null) {
			buttons[EDIT_SETTINGS] = (JToggleButton) add(createButton(IconLib.getEditSettingsIcon(), listeners[EDIT_SETTINGS]));
			if (buttonController != null) {
				buttonController.registerButtonOfType(buttons[EDIT_SETTINGS], buttonController.EDIT_SETTINGS);
			}
			buttons[EDIT_SETTINGS].setToolTipText("Open/Close settings tab");
		}
		if (listeners[HELP] != null) {
			buttons[HELP] = (JToggleButton) add(createButton(IconLib.getHelpIcon(), listeners[HELP]));
			buttons[HELP].setToolTipText("Open help window");
		}

		if (listeners[QUIT] != null) {
			buttons[QUIT] = (JToggleButton) add(createButton(IconLib.getQuitIcon(), listeners[QUIT]));
			buttons[QUIT].setToolTipText("Quit");
		}

		if (additionalPanel != null){
			addSeparator();
			addSeparator();
			addSeparator();
			addSeparator();
			addSeparator();
			addSeparator();
			addSeparator();
			addSeparator();
			add(additionalPanel);
		}

		for (int i = 0; i < buttons.length; i++) {
			if (buttons[i] != null) {
				buttons[i].setMaximumSize(new Dimension(35, 35));
				buttons[i].setMinimumSize(new Dimension(35, 35));
				buttons[i].setPreferredSize(new Dimension(35, 35));
				buttons[i].setBorder(BorderFactory.createRaisedBevelBorder());
			}
		}
	}

	public void setActionListener(ActionListener listener, int TYPE) {
		listeners[TYPE] = listener;
	}

	public ActionListener getActionListener(int TYPE) {
		return listeners[TYPE];
	}

	public void setAdditionalPanel(JPanel panel) {
		additionalPanel = panel;
	}

	private JToggleButton createButton(ImageIcon ICON, ActionListener LISTENER) {
		JToggleButton tmpButton = new JToggleButton(ICON);
		tmpButton.addActionListener(LISTENER);
		return tmpButton;
	}

	// return a button matching the number
	public JToggleButton getButton(int NUMBER) {
		return buttons[NUMBER];
	}
}
