/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool.components;

import searchtool.*;
import searchtool.util.*;
import searchtool.actions.*;
import searchtool.interfaces.*;
import searchtool.wrappers.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;

/**
 *
 * this is the big jtable, including a mouselistener to keep track of selected elements.
 * it also contains an inner-class-mouselistener for the header to listen for column
 * selections.
 *
 */

public class SearchtoolTable extends JTable implements TableCellRenderer, MouseListener {

	private int selected_column;
	private SelectedElementTracker model;
	private SearchtoolButtonBar buttonbar;
	private SearchtoolPopupMenu popupMenu;
	private TableModel ctm;
	private ButtonTracker buttonTracker;

	public SearchtoolTable(SelectedElementTracker STM) {
		super(STM);
		model = STM;
		model.registerTable(this);
		ctm = this.getModel();
		if (model.getListManager() != null)
			buttonTracker = model.getListManager().getButtonTracker();
		setShowGrid(false);
		setOpaque(true);
		getTableHeader().setReorderingAllowed(false);
		setSelectionBackground(Color.darkGray);
		setSelectionForeground(Color.white);
		setBackground(Searchtool.Constants.backgroundColor);
		setSelectionModel(new DefaultListSelectionModel());
		getSelectionModel().setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);

		// DEFAULT COLUMN SIZES
		getColumn(model.getColumnName(model.RESPONSE)).setPreferredWidth(30);
		getColumn(model.getColumnName(model.SERVERNAME)).setPreferredWidth(220);
		getColumn(model.getColumnName(model.IP)).setPreferredWidth(180);
		getColumn(model.getColumnName(model.MAP)).setPreferredWidth(100);
		getColumn(model.getColumnName(model.CURRENTPLAYERS)).setPreferredWidth(50);
		getColumn(model.getColumnName(model.MAXPLAYERS)).setPreferredWidth(50);
		getColumn(model.getColumnName(model.PASSWORD)).setPreferredWidth(50);

		
		// header cell renderer
		getTableHeader().setDefaultRenderer(this);
		// initially select proper column
		if (buttonTracker != null) {
			selected_column = buttonTracker.getValueAsInt(buttonTracker.SORTING_CRITERION);
		}
		// add mouse listener
		addMouseListener(this);
	}

	/*
	 * this method overrides the one of the superclass to
	 * be able to keep the selected server  ( if there is )
	 * selected , even after a table update.
	 */
	public void valueChanged(ListSelectionEvent e) {
		super.valueChanged(e);
		// NOTE: this method is used by the superclass, so model might
		// not be initialized yet !
		if (model != null)
			selectElement();
	}

	/*
	 * register the buttonbar that belongs to this table
	 */

	public void registerButtonBar(SearchtoolButtonBar BUTTONBAR) {
		buttonbar = BUTTONBAR;
	}
	public SearchtoolButtonBar getRegisteredButtonBar() {
		return buttonbar;
	}

	public void registerPopupMenu(SearchtoolPopupMenu menu) {
		popupMenu = menu;
	}
	public SearchtoolPopupMenu getRegisteredPopupMenu() {
		return popupMenu;
	}

	public void setSelectedColumn(int COLUMN) {
		selected_column = COLUMN;
	}

	/*
	 * select the element in the 'selected_element' variable
	 */
	public void selectElement() {
		int index = model.getSelectedIndex();
		if (getRowCount() > index && index >= 0) {
			setRowSelectionInterval(index, index);
			if (model != null) {
				model.fireTableRowsUpdated(index, index);
			}
		}
	}

	/*
	 *  select the given element, and if filtered out refilter
	 */
	public void selectElement(Element selectMe) {
		ElementListManager listmanager = model.getListManager();
		listmanager.doNotFilterElement(selectMe);
		listmanager.filterElementList();
		model.bufferNewLists();
		model.setSelectedElement(selectMe);
		// scrollRectToVisible does give a little trouble while sorting is in progress,
		// so i turn it off for a 'second'
		if (buttonTracker != null) {
			if (buttonTracker.getValue(ButtonTracker.SORT_BUTTON).equals(ButtonTracker.ENABLED)) {
				buttonTracker.setValue(ButtonTracker.SORT_BUTTON, ButtonTracker.DISABLED);
				scrollRectToVisible(getCellRect(model.getIndexOf(selectMe), 1, true));
				buttonTracker.setValue(ButtonTracker.SORT_BUTTON, ButtonTracker.DISABLED);
			} else {
				scrollRectToVisible(getCellRect(model.getIndexOf(selectMe), 1, true));
			}
		}
		selectElement();
	}

	/*
	 * TableCellRenderer method for the tableheader
	 */
	public Component getTableCellRendererComponent(JTable table, Object object, boolean isSelected, boolean hasFocus, int row, int column) {
		String columname = ctm.getColumnName(column);
		JLabel panel = new JLabel();
		panel.setHorizontalAlignment(panel.CENTER);
		panel.setText(columname);
		panel.setBorder(BorderFactory.createEtchedBorder());
		if (selected_column == column) {
			panel.setForeground(Color.blue);
			if (buttonTracker.getValue(ButtonTracker.REVERSE_SORT).equals(ButtonTracker.ENABLED))
				panel.setIcon(IconLib.getDownArrowIcon());
			else
				panel.setIcon(IconLib.getUpArrowIcon());
		}
		panel.setToolTipText(columname);
		return panel;
	}

	/*
	 *  mouse listener methods
	 */

	public void mouseClicked(MouseEvent e) {

		SearchtoolButtonBar buttons = getRegisteredButtonBar();

		// keep track of selected server
		int row = rowAtPoint(e.getPoint());
		model.setSelectedElement(row);
		selectElement();

		// do stuff that was bind to the doubleclick
		if (e.getClickCount() == 2) {
			if (buttons != null) {
				SearchtoolSettings settings = Searchtool.getGlobalSettings();
				if (settings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.CONNECT)
					buttons.getButton(buttons.CONNECT).doClick();
				else if (settings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.MAKE_FAVOURITE)
					buttons.getButton(buttons.MAKE_FAVOURITE).doClick();
				else if (settings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.MAKE_BLACKLIST)
					buttons.getButton(buttons.MAKE_OUTSIDER).doClick();
				else if (settings.getValueAsInt(SearchtoolSettings.DOUBLECLICK_BIND) == SearchtoolSettings.GET_DETAILS)
					buttons.getButton(buttons.DISPLAY_DETAILS).doClick();
			}
		}
	}
	public void mousePressed(MouseEvent mouseEvent) {
		if (mouseEvent.isPopupTrigger()) {
			if (getRegisteredPopupMenu() != null && model.getSelectedElement() != null)
				getRegisteredPopupMenu().show(this, mouseEvent.getX(), mouseEvent.getY());
		}
	}
	public void mouseReleased(MouseEvent mouseEvent) {
		if (mouseEvent.isPopupTrigger()) {
			if (getRegisteredPopupMenu() != null && model.getSelectedElement() != null)
				getRegisteredPopupMenu().show(this, mouseEvent.getX(), mouseEvent.getY());
		}
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}
}
