/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package searchtool.util;

import searchtool.wrappers.*;
import searchtool.*;
import searchtool.interfaces.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
// third party
import com.jclark.xml.parse.*;
import com.jclark.xml.parse.base.*;
import com.jclark.xml.parse.io.ApplicationImpl;
import com.jclark.xml.output.*;
import gnu.trove.*;

/**
 * @author thorsten
 *
 *  an object of this class is supposed to make sure that there is only one object of
 *  Element or subclasses for one server (identified by ip and port)  in all the lists.
 *  
 * an object ofElement or its subclasses will be added to the given list
 * by using either the given object or an already existing one.
 *  
 * also this class makes sure not to add the same object to the same list twice.
 * 
 * adding an object to the blacklist removes it from any other list.
 *  
 * adding an object to the favourites removes it from the blacklist, but keeps it in the mainlist.
 * 
 * removing an object from the blacklist re-adds it to the mainlist.
 * 
 * lastconnectedlist is pretty much independent from the other lists, but still
 * it uses the same objects.
 * 
 * 
 * last but not least this class takes care of loading and saving of the one-and-only
 * serverlist.
 * 
 */
public class ElementController {

	public static final int MAIN_LIST = 0, FAVOURITE_LIST = 1, BLACK_LIST = 2, CONNECTED_LIST = 3, RESULT_LIST = 5;

	private ServerListManager[] listManager = new ServerListManager[5];
	// the game controller
	private GameController gameController = null;
	// the file that stores the complete list of servers
	private File serverfile = null;
	// the complete list of servers
	private TLongObjectHashMap serverlist = new TLongObjectHashMap();
	// the complete list of players(searchresults).
	private TLongObjectHashMap playerlist = new TLongObjectHashMap();
	// a progressbar that can be used to display loading status
	private JProgressBar registeredProgressBar = null;

	public ElementController(GameController gameController, File serverfile) {
		this.gameController = gameController;
		this.serverfile = serverfile;
	}

	public void destroy() {
		listManager = null;
		gameController = null;
		serverfile = null;
		serverlist = null;
		playerlist = null;
		registeredProgressBar = null;
	}

	/*
	 *  load the complete list of servers from file.
	 */
	public synchronized void loadServerList(JProgressBar progressBar) {
		try {
			ServerElement.ListDataElementParserCallback xmlParser = gameController.getXMLParser();
			if (progressBar != null)
				xmlParser.registerProgressBar(progressBar);
			EntityManagerImpl entityManagerImpl = new EntityManagerImpl();
			OpenEntity openEntity = entityManagerImpl.openFile(serverfile.getAbsolutePath());
			ParserImpl parserImpl = new ParserImpl();
			parserImpl.setApplication(xmlParser);
			parserImpl.setEntityManager(entityManagerImpl);
			parserImpl.setLocale(Locale.getDefault());
			parserImpl.parseDocument(openEntity);
			Vector tmpServerList = xmlParser.getLoadedList();
			for (int i = 0; i < tmpServerList.size(); i++) {
				addElement((ServerElement) tmpServerList.get(i));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 *  save the complete list of servers to file (not the SearchtoolPlayer objects)
	 */
	public synchronized void saveServerList() {
		try {
			ServerElement.ListDataElementXMLWriter xmlWriter = gameController.getXMLWriter();
			int size = serverlist.size();
			Object[] values = serverlist.getValues();
			xmlWriter.startOutput(size);
			for (int i = 0; i < values.length; i++) {
				xmlWriter.startListElement();
				xmlWriter.writeElement((ServerElement) values[i]);
				xmlWriter.endListElement();
			}
			xmlWriter.endOutput();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 *  register a new listmanager
	 */
	public synchronized void registerListManager(ServerListManager listmanager, int LISTTYPE) {
		listManager[LISTTYPE] = listmanager;
	}

	/*
	 *  clear the contents of the list of the given type
	 */
	public synchronized void clearList(int LISTTYPE) {
		if (listManager[LISTTYPE] != null)
			listManager[LISTTYPE].setCompleteList(new ServerElement[0]);
		// now update the complete serverlist
		serverlist.clear();
		if (MAIN_LIST != LISTTYPE) {
			Element[] keepUs = listManager[MAIN_LIST].getCompleteList();
			for (int i = 0; i < keepUs.length; i++) {
				serverlist.put(keepUs[i].getId(), keepUs[i]);
			}
		}
		if (FAVOURITE_LIST != LISTTYPE) {
			Element[] keepUs = listManager[FAVOURITE_LIST].getCompleteList();
			for (int i = 0; i < keepUs.length; i++) {
				serverlist.put(keepUs[i].getId(), keepUs[i]);
			}
		}
		if (BLACK_LIST != LISTTYPE) {
			Element[] keepUs = listManager[BLACK_LIST].getCompleteList();
			for (int i = 0; i < keepUs.length; i++) {
				serverlist.put(keepUs[i].getId(), keepUs[i]);
			}
		}
		if (CONNECTED_LIST != LISTTYPE) {
			Element[] keepUs = listManager[CONNECTED_LIST].getCompleteList();
			for (int i = 0; i < keepUs.length; i++) {
				serverlist.put(keepUs[i].getId(), keepUs[i]);
			}
		}
	}

	/**
	 *  return the contents of  a list of the given type
	 */
	public synchronized TLongObjectHashMap getElementList(int TYPE) {
		if (listManager[TYPE] != null) {
			TLongObjectHashMap stayMap = listManager[TYPE].getCompleteListAsMap();
			TLongObjectHashMap returnMap = new TLongObjectHashMap(stayMap.size());
			long[] keys = stayMap.keys();
			for (int i = 0; i < keys.length; i++)
				returnMap.put(keys[i], stayMap.get(keys[i]));
			return returnMap;
		} else
			return new TLongObjectHashMap(0);
	}

	/*
	 *  return a complete list of available servers.
	 */
	public synchronized TLongObjectHashMap getCompleteServerList() {
		TLongObjectHashMap stayMap = serverlist;
		TLongObjectHashMap returnMap = new TLongObjectHashMap(stayMap.size());
		long[] keys = stayMap.keys();
		for (int i = 0; i < keys.length; i++)
			returnMap.put(keys[i], stayMap.get(keys[i]));
		return returnMap;
	}

	/*
	 *  return a ServerElement for the given IpWrapper, if there is one.
	 */
	public synchronized ServerElement getServerElementFor(IpWrapper ipWrapper) {
		Object returnMe = serverlist.get(ipWrapper.id);
		if (returnMe == null)
			return null;
		else
			return (ServerElement) returnMe;
	}

	/*
	 * remove the given element from any list.
	 */
	public synchronized void removeElement(ServerElement REMOVEME) {

		if (serverlist.containsKey(REMOVEME.getId())) {
			serverlist.remove(REMOVEME.getId());
		}

		// now remove the reference from any list 
		if (listManager[MAIN_LIST] != null) {
			listManager[MAIN_LIST].removeElement(REMOVEME);
		}
		if (listManager[FAVOURITE_LIST] != null) {
			listManager[FAVOURITE_LIST].removeElement(REMOVEME);
		}
		if (listManager[BLACK_LIST] != null) {
			listManager[BLACK_LIST].removeElement(REMOVEME);
		}
		if (listManager[CONNECTED_LIST] != null) {
			listManager[CONNECTED_LIST].removeElement(REMOVEME);
		}
	}

	/*
	 *  selfexplaining
	 */
	public synchronized boolean isInList(ServerElement AMIHERE, int TYPE) {
		if (listManager[TYPE] != null) {
			return listManager[TYPE].isElementInList(AMIHERE);
		} else
			return false;
	}

	/*
	 *  using this method to add an object of type Element to any one of the 
	 *  possible lists.  the method will determine by type and flags which list
	 *  is the proper one.
	 * 
	 */
	public synchronized void addElement(ServerElement ADDME) {

		ServerElement reference = (ServerElement) ADDME;
		// add to global list
		serverlist.put(((ServerElement) reference).getId(), reference);
		// check connected list
		if (listManager[CONNECTED_LIST] != null) {
			if (reference.isLastConnectedlist()) {
				listManager[CONNECTED_LIST].addElement(reference);
				this.limitLastConnectedList(30);
			} else if (listManager[CONNECTED_LIST].isElementInList(reference)) {
				listManager[CONNECTED_LIST].removeElement(reference);
			}
		}
		// check blacklist
		if (listManager[BLACK_LIST] != null) {
			if (reference.isBlacklist())
				listManager[BLACK_LIST].addElement(reference);
			else if (listManager[BLACK_LIST].isElementInList(reference))
				listManager[BLACK_LIST].removeElement(reference);
		}
		// check mainlist
		if (listManager[MAIN_LIST] != null) {
			if (!reference.isBlacklist())
				listManager[MAIN_LIST].addElement(reference);
			else if (listManager[MAIN_LIST].isElementInList(reference))
				listManager[MAIN_LIST].removeElement(reference);
		}
		// check favouritelist
		if (listManager[FAVOURITE_LIST] != null) {
			if (reference.isFavourite()) {
				listManager[FAVOURITE_LIST].addElement(reference);
			}
			if (!reference.isFavourite())
				if (listManager[FAVOURITE_LIST].isElementInList(reference))
					listManager[FAVOURITE_LIST].removeElement(reference);
		}
	}

	public void limitLastConnectedList(int limit) {
		TLongObjectHashMap map = this.getElementList(this.CONNECTED_LIST);
		long[] keys = map.keys();
		if (keys.length < limit)
			return;
		ServerElement minimum = (ServerElement) map.get(keys[0]);
		for (int i = 1; i < keys.length; i++) {
			//System.out.println(minimum.getDate());
			if (((ServerElement) map.get(keys[i])).getDate().getTime() < minimum.getDate().getTime()) {
				minimum = (ServerElement) map.get(keys[i]);
			}
		}
		minimum.setLastConnectedlist(false);
	}
}
